@@ -18,15 +18,19 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from __future__ import print_function
+
 from datetime import datetime, date, time, timedelta
 from decimal import Decimal
-import cPickle as pickle
+from functools import partial
 import re
-try:
-    import uuid
-except ImportError:
-    uuid = None
+import uuid
+import weakref
 
+import six
+from six.moves import cPickle as pickle
+
+from storm.compat import json
 from storm.exceptions import NoneError
 from storm import Undef, has_cextensions
 
@@ -39,6 +43,7 @@ __all__ = [
     "IntVariable",
     "FloatVariable",
     "DecimalVariable",
+    "BytesVariable",
     "RawStrVariable",
     "UnicodeVariable",
     "DateTimeVariable",
@@ -48,18 +53,26 @@ __all__ = [
     "EnumVariable",
     "UUIDVariable",
     "PickleVariable",
+    "JSONVariable",
     "ListVariable",
 ]
 
 
+if six.PY3:
+    _buffer_type = memoryview
+else:
+    _buffer_type = buffer
+
+
 class LazyValue(object):
     """Marker to be used as a base class on lazily evaluated values."""
     __slots__ = ()
 
 
-def raise_none_error(column):
+def raise_none_error(column, default=False):
+    description = "default value" if default else "value"
     if not column:
-        raise NoneError("None isn't acceptable as a value")
+        raise NoneError("None isn't acceptable as a %s" % description)
     else:
         from storm.expr import compile, CompileError
         name = column.name
@@ -69,25 +82,11 @@ def raise_none_error(column):
                 name = "%s.%s" % (table, name)
             except CompileError:
                 pass
-        raise NoneError("None isn't acceptable as a value for %s" % name)
+        raise NoneError(
+            "None isn't acceptable as a %s for %s" % (description, name))
 
 
-def VariableFactory(cls, **old_kwargs):
-    """Build cls with kwargs of constructor updated by kwargs of call.
-
-    This is really an implementation of partial/curry functions, and
-    is replaced by 'partial' when 2.5+ is in use.
-    """
-    def variable_factory(**new_kwargs):
-        kwargs = old_kwargs.copy()
-        kwargs.update(new_kwargs)
-        return cls(**kwargs)
-    return variable_factory
-
-try:
-    from functools import partial as VariableFactory
-except ImportError:
-    pass
+VariableFactory = partial
 
 
 class Variable(object):
@@ -136,12 +135,14 @@ class Variable(object):
         @type column: L{storm.expr.Column}
         @param column: The column that this variable represents. It's
             used for reporting better error messages.
-        @type event: L{EventSystem}
+        @type event: L{storm.event.EventSystem}
         @param event: The event system to broadcast messages with. If
             not specified, then no events will be broadcast.
         """
         if not allow_none:
             self._allow_none = False
+            if value is None:
+                raise_none_error(column, default=True)
         if value is not Undef:
             self.set(value, from_db)
         elif value_factory is not Undef:
@@ -151,7 +152,7 @@ class Variable(object):
             self._validator_object_factory = validator_object_factory
             self._validator_attribute = validator_attribute
         self.column = column
-        self.event = event
+        self.event = weakref.proxy(event) if event is not None else None
 
     def get_lazy(self, default=None):
         """Get the current L{LazyValue} without resolving its value.
@@ -327,7 +328,7 @@ class BoolVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, (int, long, float, Decimal)):
+        if not isinstance(value, (six.integer_types, float, Decimal)):
             raise TypeError("Expected bool, found %r: %r"
                             % (type(value), value))
         return bool(value)
@@ -337,7 +338,7 @@ class IntVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, (int, long, float, Decimal)):
+        if not isinstance(value, (six.integer_types, float, Decimal)):
             raise TypeError("Expected int, found %r: %r"
                             % (type(value), value))
         return int(value)
@@ -347,7 +348,7 @@ class FloatVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, (int, long, float, Decimal)):
+        if not isinstance(value, (six.integer_types, float, Decimal)):
             raise TypeError("Expected float, found %r: %r"
                             % (type(value), value))
         return float(value)
@@ -358,8 +359,8 @@ class DecimalVariable(Variable):
 
     @staticmethod
     def parse_set(value, from_db):
-        if (from_db and isinstance(value, basestring) or
-            isinstance(value, (int, long))):
+        if (from_db and isinstance(value, six.string_types) or
+            isinstance(value, six.integer_types)):
             value = Decimal(value)
         elif not isinstance(value, Decimal):
             raise TypeError("Expected Decimal, found %r: %r"
@@ -369,28 +370,32 @@ class DecimalVariable(Variable):
     @staticmethod
     def parse_get(value, to_db):
         if to_db:
-            return str(value)
+            return six.text_type(value)
         return value
 
 
-class RawStrVariable(Variable):
+class BytesVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if isinstance(value, buffer):
-            value = str(value)
-        elif not isinstance(value, str):
-            raise TypeError("Expected str, found %r: %r"
+        if isinstance(value, _buffer_type):
+            value = bytes(value)
+        elif not isinstance(value, bytes):
+            raise TypeError("Expected bytes, found %r: %r"
                             % (type(value), value))
         return value
 
 
+# DEPRECATED: BytesVariable was RawStrVariable until 0.22.
+RawStrVariable = BytesVariable
+
+
 class UnicodeVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, unicode):
-            raise TypeError("Expected unicode, found %r: %r"
+        if not isinstance(value, six.text_type):
+            raise TypeError("Expected text, found %r: %r"
                             % (type(value), value))
         return value
 
@@ -406,7 +411,7 @@ class DateTimeVariable(Variable):
         if from_db:
             if isinstance(value, datetime):
                 pass
-            elif isinstance(value, (str, unicode)):
+            elif isinstance(value, six.string_types):
                 if " " not in value:
                     raise ValueError("Unknown date/time format: %r" % value)
                 date_str, time_str = value.split(" ")
@@ -420,11 +425,22 @@ class DateTimeVariable(Variable):
                 else:
                     value = value.astimezone(self._tzinfo)
         else:
-            if type(value) in (int, long, float):
+            if type(value) in six.integer_types + (float, ):
                 value = datetime.utcfromtimestamp(value)
             elif not isinstance(value, datetime):
                 raise TypeError("Expected datetime, found %s" % repr(value))
             if self._tzinfo is not None:
+                # Python 3.6 gained support for calling the astimezone
+                # method on naive datetime objects, presuming them to
+                # represent system local time.  This is probably
+                # inappropriate for most uses of Storm, since depending on
+                # what the system local time happens to be is usually a
+                # mistake, so forbid this explicitly for now; we can always
+                # open it up later if there's a good reason.
+                if (value.tzinfo is None or
+                        value.tzinfo.utcoffset(value) is None):
+                    raise ValueError(
+                        "Expected aware datetime, found naive: %r" % value)
                 value = value.astimezone(self._tzinfo)
         return value
 
@@ -436,9 +452,11 @@ class DateVariable(Variable):
         if from_db:
             if value is None:
                 return None
+            if isinstance(value, datetime):
+                return value.date()
             if isinstance(value, date):
                 return value
-            if not isinstance(value, (str, unicode)):
+            if not isinstance(value, six.string_types):
                 raise TypeError("Expected date, found %s" % repr(value))
             if " " in value:
                 value, time_str = value.split(" ")
@@ -461,7 +479,7 @@ class TimeVariable(Variable):
                 return None
             if isinstance(value, time):
                 return value
-            if not isinstance(value, (str, unicode)):
+            if not isinstance(value, six.string_types):
                 raise TypeError("Expected time, found %s" % repr(value))
             if " " in value:
                 date_str, value = value.split(" ")
@@ -484,7 +502,7 @@ class TimeDeltaVariable(Variable):
                 return None
             if isinstance(value, timedelta):
                 return value
-            if not isinstance(value, (str, unicode)):
+            if not isinstance(value, six.string_types):
                 raise TypeError("Expected timedelta, found %s" % repr(value))
             return _parse_interval(value)
         else:
@@ -497,8 +515,7 @@ class UUIDVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        assert uuid is not None, "The uuid module was not found."
-        if from_db and isinstance(value, basestring):
+        if from_db and isinstance(value, six.string_types):
             value = uuid.UUID(value)
         elif not isinstance(value, uuid.UUID):
             raise TypeError("Expected UUID, found %r: %r"
@@ -507,7 +524,7 @@ class UUIDVariable(Variable):
 
     def parse_get(self, value, to_db):
         if to_db:
-            return str(value)
+            return six.text_type(value)
         return value
 
 
@@ -550,10 +567,10 @@ class MutableValueVariable(Variable):
         Variable.__init__(self, *args, **kwargs)
         if self.event is not None:
             self.event.hook("start-tracking-changes", self._start_tracking)
-            self.event.hook("object-deleted", self._detect_changes)
+            self.event.hook("object-deleted", self._detect_changes_and_stop)
 
     def _start_tracking(self, obj_info, event_system):
-        self._event_system = event_system
+        self._event_system = weakref.proxy(event_system)
         self.event.hook("stop-tracking-changes", self._stop_tracking)
 
     def _stop_tracking(self, obj_info, event_system):
@@ -565,6 +582,11 @@ class MutableValueVariable(Variable):
             self.get_state() != self._checkpoint_state):
             self.event.emit("changed", self, None, self._value, False)
 
+    def _detect_changes_and_stop(self, obj_info):
+        self._detect_changes(obj_info)
+        if self._event_system is not None:
+            self._stop_tracking(obj_info, self._event_system)
+
     def get(self, default=None, to_db=False):
         if self._event_system is not None:
             self._event_system.hook("flush", self._detect_changes)
@@ -579,29 +601,62 @@ class MutableValueVariable(Variable):
         super(MutableValueVariable, self).set(value, from_db)
 
 
-class PickleVariable(MutableValueVariable):
+class EncodedValueVariable(MutableValueVariable):
+
     __slots__ = ()
 
     def parse_set(self, value, from_db):
         if from_db:
-            if isinstance(value, buffer):
-                value = str(value)
-            return pickle.loads(value)
+            if isinstance(value, _buffer_type):
+                value = bytes(value)
+            return self._loads(value)
         else:
             return value
 
     def parse_get(self, value, to_db):
         if to_db:
-            return pickle.dumps(value, -1)
+            return self._dumps(value)
         else:
             return value
 
     def get_state(self):
-        return (self._lazy_value, pickle.dumps(self._value, -1))
+        return (self._lazy_value, self._dumps(self._value))
 
     def set_state(self, state):
         self._lazy_value = state[0]
-        self._value = pickle.loads(state[1])
+        self._value = self._loads(state[1])
+
+
+class PickleVariable(EncodedValueVariable):
+
+    def _loads(self, value):
+        return pickle.loads(value)
+
+    def _dumps(self, value):
+        return pickle.dumps(value, -1)
+
+
+class JSONVariable(EncodedValueVariable):
+
+    __slots__ = ()
+
+    def _loads(self, value):
+        if not isinstance(value, six.text_type):
+            raise TypeError(
+                "Cannot safely assume encoding of byte string %r." % value)
+        return json.loads(value)
+
+    def _dumps(self, value):
+        # http://www.ietf.org/rfc/rfc4627.txt states that JSON is text-based
+        # and so we treat it as such here. In other words, this method returns
+        # Unicode text and never bytes.
+        dump = json.dumps(value, ensure_ascii=False)
+        if not isinstance(dump, six.text_type):
+            # json.dumps() does not always return unicode. See
+            # http://code.google.com/p/simplejson/issues/detail?id=40 for one
+            # of many discussions of str/unicode handling in simplejson.
+            dump = dump.decode("utf-8")
+        return dump
 
 
 class ListVariable(MutableValueVariable):
